#ifndef _packet_h_
#define _packet_h_

#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

struct Packet{
  unsigned ihl:4;
  unsigned version:4;
  unsigned tos:8;
  unsigned len:16;
  unsigned id:16;
  unsigned frag:16;
  unsigned ttl:8;
  unsigned prot:8;
  unsigned crc:16;
  unsigned source:32;
  unsigned dest:32;
  char data;
};

typedef Packet* Packetp;

/* IP options 
   
   The option field is variable in length.  There may be zero or more
    options.  There are two cases for the format of an option:

      Case 1:  A single octet of option-type.

      Case 2:  An option-type octet, an option-length octet, and the
               actual option-data octets.

    The option-length octet counts the option-type octet and the
    option-length octet as well as the option-data octets.

    The option-type octet is viewed as having 3 fields:

      1 bit   copied flag,
      2 bits  option class,
      5 bits  option number.

    The copied flag indicates that this option is copied into all
    fragments on fragmentation.

      0 = not copied
      1 = copied

    The option classes are:

      0 = control
      1 = reserved for future use
      2 = debugging and measurement
      3 = reserved for future use

   Router Alert:

   The Router Alert option has the following format:

                 +--------+--------+--------+--------+
                 |10010100|00000100|  2 octet value  |
                 +--------+--------+--------+--------+

   Type:
     Copied flag:  1 (all fragments must carry the option)
     Option class: 0 (control)
     Option number: 20 (decimal)

   Length: 4

   Value:  A two octet code with the following values:
     0 - Router shall examine packet
     1-65535 - Reserved


*/

#define IPOPT_NUMBER(a)   ( a&0x1f )
#define IPOPT_CLASS(a)    ( (a&0x60) >> 5 )
#define IPOPT_COPY(a)    ( (a&0x80) >> 7 )

// option classes

#define IPOPT_CLASS_CONTROL           0
#define IPOPT_CLASS_RESERVED1         1
#define IPOPT_CLASS_MEASUREMENT       2
#define IPOPT_CLASS_RESERVED2         3

// option numbers

#define IPOPT_END       0 
#define IPOPT_NOOP      1 
#define IPOPT_SEC       2 
#define IPOPT_LSRR      3 
#define IPOPT_TIMESTAMP 4 
#define IPOPT_RR        7 
#define IPOPT_SID       8 
#define IPOPT_SSRR      9 
#define IPOPT_RA        20

#define PKT_LENGTH(P)  ( ntohs((P)->len) )
#define PKT_DEST(P)    ( (P)->dest )
#define PKT_SOURCE(P)  ( (P)->source )
#define PKT_ID(P)      ( (P)->id )
#define PKT_TTL(P)     ( (P)->ttl )
#define PKT_DEC_TTL(P) ( ++(P)->ttl )
#define PKT_PROTO(P)   ( (P)->prot ) 
#define PKT_DATA(P)    ( ((char*)(P)) + ((P)->ihl*4) )
#define PKT_TOS(P)     ( (P)->tos ) 
#define PKT_NOPTS(P)   ( (P)->ihl*4-20)
#define PKT_OPTS(P)    ( ((char*)(P)) + 20 )
#define PKT_IHL(P)     ( (P)->ihl )
#define PKT_FRAG(P)    ( (P)->frag )
#define PKT_FOFF(P)    ( (ntohs((P)->frag)&(0xffff-7*8192))<<3 )
#define PKT_MF(P)      ( ((P)->frag&0x20)>>5 )
#define PKT_DF(P)      ( ((P)->frag&0x40)>>6 )
#define PKT_DATALEN(P) ( ntohs((P)->len) - ((P)->ihl*4) )
#define PKT_HDRLEN(P)  ( (P)->ihl*4 ) 


#endif



